
#include <stdlib.h>
#include <stdio.h>

#ifndef __TURBOC__
  #include <malloc.h>
#else
  #include <alloc.h>
#endif

#include "cgdk.h"
#include "dpmi.h"

unsigned long _mallocated;
unsigned long _mtotal;

int _allocCount;
int _freeCount;

void _mexit (const char *exitStr, int requestedMemorySize)
{
  unsigned long mtotal=_mtotal;
  unsigned long mallocated=_mallocated;
  unsigned long mfree=_mfree;

  if (exitStr)
  {
    _cexit ();

    puts (exitStr);

    printf ("\n");
    printf ("Requested memory block size : %8lu bytes\n", requestedMemorySize);
    printf ("Memory available total      : %8lu bytes\n", mtotal);
    printf ("Memory allocated            : %8lu bytes\n", mallocated);
    printf ("Memory free                 : %8lu bytes\n", mfree);
    printf ("\n");

    exit (EXIT_FAILURE);
  }
}

void _mdone (void)
{
  _minit ();
}

void _minit (void)
{
//  _atexit (_mdone);
  if (_mtotal == 0)
  {
    _mtotal = DPMI_maxAvail;
  }
}

void *_alloc (int size)
{
  void *mem;

  mem = malloc (size);
  if (mem)
  {
    _allocCount++;
    #ifndef __TURBOC__
      size = _msize (mem);
      _mallocated += size;

//      if (size == 12)
//         DPMI_debugInt ();

    #else
      _mallocated = _mtotal - coreleft ();
    #endif
  }
  return mem;
}


void *_malloc (int size, char *errorStr)
{
  void *mem;
//  allocs++;

  mem = malloc (size);
  if (mem)
  {
    _allocCount++;
    #ifndef __TURBOC__
      size = _msize (mem);
      _mallocated += size;

//     if (size == 12)
//        DPMI_debugInt ();

    #else
      _mallocated = _mtotal - coreleft ();
    #endif
  }
  else
  {
    _mexit (errorStr, size);
  }
  return mem;
}

void *_calloc (int n, int size, char *errorStr)
{
  void *mem;

  mem = calloc (n, size);
  if (mem)
  {
    _allocCount++;
    #ifndef __TURBOC__
      size = _msize (mem);
      _mallocated += size;

//      if (size == 12)
//         DPMI_debugInt ();

    #else
      _mallocated = _mtotal - coreleft ();
    #endif
  }
  else
  {
    _mexit (errorStr, n*size);
  }
  return mem;
}

void *_realloc (void *mem, int newsize, char *errorStr)
{
  int size;

  if (mem)
  {
    #ifndef __TURBOC__
      size = _msize (mem);
      mem = realloc (mem, newsize);
      if (mem == NULL)
      {
        _mexit (errorStr, newsize - size);
        return NULL;
      }
      size -= _msize (mem);
      _mallocated -= size;
    #else
      mem = realloc (mem, newsize);
      _mallocated = _mtotal - coreleft ();
    #endif

    return mem;
  }
  else
  {
    _mexit (errorStr, size);
    return NULL;
  }
}

void _free (void *mem)
{
  if (mem)
  {
    _freeCount++;
    #ifndef __TURBOC__
      int size;

      size = _msize (mem);
      if (size != -1)
      {
        free (mem);
        _mallocated -= size;
      }
    #else
      free (mem);
      _mallocated = _mtotal - coreleft ();
    #endif
  }
//  else
//  {
//  CGDK_exit ("Attempt to free NULL memory block.\n", EXIT_FAILURE);
//  }
}
