
#include <mem.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "pal.h"
#include "car.h"
#include "svga.h"
#include "lin.h"
#include "csf.h"
#include "cgdk.h"
#include "dpmi.h"
#include "file.h"

#include "csffont_.cpp"

/*
void *CSF_raw (TCSF *csf, int n)
{
  if (csf)
  {
    n = n % csf->count;
    return ((char *)csf->raw+n*csf->size);
  }
  return NULL;
}
*/

int CSF_changeBaseColor (TCSF *csf, long newBaseColor)
{
  long i;
  long maxColors;

  if (csf != NULL)
  {
    maxColors = 2 << (8*csf->bytesPerPixel-1);
    if ((newBaseColor > 0) && (newBaseColor+csf->maxColor <= maxColors))
    {
      for (i = 0; i < csf->size*csf->count; i++)
      {
        ((char *) csf->raw) [i] += (newBaseColor - csf->baseColor);
      }
      csf->baseColor = newBaseColor;
      return 0;
    }
  }
  return 1;
}


void CSF_cpy (TCSF *csf, void *Dst,
                 int Dst_X, int Dst_Y, int DstsizeX, int DstsizeY, int n)
{
  LIN_movSpr (Dst, Dst_X, Dst_Y, DstsizeX, DstsizeY,
              csf->raw+n*csf->size, csf->sizeX, csf->sizeX, csf->sizeY);
}


TCSF *CSF_new (TCAR *carHandle, const char *csfiteName)
{
  TCSF *csf;
  int colorsSize;

  if (sizeof (TCSF) == CSF_HEADERSIZE)
    ;
  else
    return NULL;

  /* nalezeni csfitu */
  if (CAR_seek (carHandle, csfiteName, 0) == -1)
    return NULL;

  /* naalokovani bufferu pro header csfitu */
  if ((csf = (TCSF *) _alloc (sizeof (TCSF))) == NULL)
  {
    return NULL;
  }

  /* nacteni hlavicky csfitu */
  if (CAR_read (carHandle, csf, sizeof (TCSF)) != sizeof (TCSF))
  {
    _free (csf);
    return NULL;
  }

  if ((csf->id != CSF_ID) || (csf->version > CSF_VERSION) || (strcmp (csf->desc, CSF_DESC) != 0))
  {
    _free (csf);
    return NULL;
  }

  if (csf->maxChar != 0)
  {
    // nacteni char table
    csf->charTable = (long *) _alloc (sizeof (long)*csf->maxChar);
    if (csf->charTable)
    {
      if (CAR_read (carHandle, csf->charTable, sizeof (long)*csf->maxChar) != sizeof(long)*csf->maxChar)
      {
        _free (csf->charTable);
        _free (csf);
        return NULL;
      }
    }
    else
    {
      _free (csf);
      return NULL;
    }
  }
  else
  {
    csf->charTable = NULL;
  }

  colorsSize = csf->maxColor*sizeof (TRGBColor);

  /* naalokovani pameti pro colors */
  if (colorsSize != 0)
  {
    if ((csf->colors = (TRGBColor *) _alloc (colorsSize)) == NULL)
    {
      if (csf->charTable)
        _free (csf->charTable);
      _free (csf);
      return NULL;
    }

    /* nacteni colors */
    if (CAR_read (carHandle, csf->colors, colorsSize) != colorsSize)
    {
      _free (csf->colors);
      if (csf->charTable)
        _free (csf->charTable);
      _free (csf);

      return NULL;
    }
  }
  else
  {
    csf->colors = NULL;
  }

//  csf->size = (unsigned)(csf->sizeX*csf->sizeY*csf->bytesPerPixel);

  /* naalokovani pameti pro csfite */
  if ((csf->raw = (char *) _alloc (csf->size*csf->count)) == NULL)
  {
    CSF_delete (csf);
    return NULL;
  }

  /* nacteni raw */
  if ((unsigned) CAR_read (carHandle, csf->raw, csf->size*csf->count) !=
      csf->size*csf->count)
  {
    CSF_delete (csf);
    return NULL;
  }


  return csf;
}

TCSF *CSF_dup (TCSF *csf)
{
  TCSF *csf2;
  int colorsSize;

  if (csf == NULL)
    return csf;

  /* naalokovani bufferu pro header csfitu */
  if ((csf2 = (TCSF *) _alloc (sizeof (TCSF))) == NULL)
  {
    return NULL;
  }

  memcpy (csf2, csf, sizeof (TCSF));

  if (csf2->maxChar != 0)
  {
    // nacteni char table
    csf2->charTable = (long *) _alloc (sizeof (long)*csf2->maxChar);
    if (csf2->charTable)
    {
      memcpy (csf2->charTable, csf->charTable, sizeof(long)*csf->maxChar);
    }
    else
    {
      _free (csf2);
      return NULL;
    }
  }
  else
  {
    csf2->charTable = NULL;
  }

  colorsSize = csf2->maxColor*sizeof (TRGBColor);

  /* naalokovani pameti pro colors */
  if (csf2->colors != NULL)
  {
    if ((csf2->colors = (TRGBColor *) _alloc (colorsSize)) == NULL)
    {
      if (csf2->charTable)
        _free (csf2->charTable);
      _free (csf2);
      return NULL;
    }

    /* nacteni colors */
    memcpy (csf2->colors, csf->colors, colorsSize);
  }
  else
  {
    csf2->colors = NULL;
  }

  /* naalokovani pameti pro csfite */
  if ((csf2->raw = (char *) _alloc (csf2->size*csf2->count)) == NULL)
  {
    CSF_delete (csf2);
    return NULL;
  }

  /* nacteni raw */
  memcpy (csf2->raw, csf->raw, csf->size*csf->count);

  return csf2;
}



void CSF_delete (TCSF *csf)
{
  if (csf != NULL)
  {
    if (csf->raw)
      _free (csf->raw);
    if (csf->colors)
      _free (csf->colors);
    if (csf->charTable)
      _free (csf->charTable);
    _free (csf);
  }
}

void CSF_setPal (TCSF *csf, TRGBColor *Colors, int FirstColor, int NumColors)
{
  if ((csf != NULL) && (csf->maxColor != 0))
  {
    memcpy (&(csf->colors [FirstColor]), Colors, NumColors*sizeof (TRGBColor));
  }
}


int CSF_updateColors (TCSF *csf, TRGBColor *pal, int baseColor, int palLen)
{
  int color;
  long i;
  unsigned char xlatTable [256];

  xlatTable [0] = 0;
  if ((csf != NULL) && (csf->maxColor != 0))
  {
    for (i = 0; i < csf->maxColor; )
    {
      color = PAL_insertColor (csf->colors [i], pal+baseColor, palLen);
      i++;
      if (color == -1)
      {
        xlatTable [i] = 0;
      }
      else
        xlatTable [i] = baseColor+color;
    }

    for (i = 0; i < csf->size*csf->count; i++)
    {
      color = ((unsigned char *)csf->raw) [i];
      ((unsigned char *) csf->raw) [i] = xlatTable [color];
    }
  }
  return 0;
}


#define CSF_CREATE_PHASE_1 15
#define CSF_CREATE_PHASE_2 35
#define CSF_CREATE_PHASE_MAX 50

extern TCSF *CSF_create (void *rawBuf, int sizeX, int sizeY, TRGBColor *palette,
                 int n, long charTable [256])
{
  TCSF *csf;
  char curColor;
  long i,j,a;
  long colBuf [256] = {0};
  long c;

  csf = (TCSF *) _calloc (sizeof (TCSF), 1, NULL);
  if (csf == NULL)
    return NULL;

  csf->raw = (char *) _calloc (sizeX * sizeY *n, 1, NULL);
  if (csf->raw == NULL)
  {
    _free (csf);
    return NULL;
  }

  if (charTable != NULL)
  {
    csf->maxChar = 256;
    csf->charTable = charTable;
  }
  strcpy (csf->desc, CSF_DESC);

  csf->id = CSF_ID;
  csf->version = CSF_VERSION;
  csf->sizeX = sizeX;
  csf->sizeY = sizeY;
  csf->bytesPerPixel = 1;
  csf->size = (long)sizeX*sizeY*csf->bytesPerPixel;
  csf->count = n;
  csf->baseColor = 1;

  if (palette != NULL)
  {
    curColor = 0;

    csf->colors = (TRGBColor *) _alloc (256*sizeof (TRGBColor));
    if (csf->colors == NULL)
    {
      _free (csf->raw);
      _free (csf);
      return NULL;
    }

    for (i = 0; i < (unsigned)sizeX*sizeY*n; i++)
    {
      c = ((unsigned char *)rawBuf) [i];
      if (c)
      {
        if (colBuf [c] == 0)
        {
          colBuf [c] = curColor + csf->baseColor;
          csf->colors [curColor] = palette [c];
          curColor++;
        }
        csf->raw [i] = colBuf [c];
      }
    }
    csf->maxColor = curColor;

    if (csf->maxColor == 0)
    {
      _free (csf->colors);
      csf->colors = NULL;
    }
    else
    {
      csf->colors = (TRGBColor *) _realloc ((void *)csf->colors, sizeof (TRGBColor)*csf->maxColor, NULL);
    }
  }
  else
  {
    csf->colors = NULL;
    csf->maxColor = 0;
  }

  return csf;
}


int CSF_save (TCSF *csf, const char *fileName)
{
  if (F_write (fileName, csf, sizeof (TCSF)) == 0)
  {
    F_append (fileName, csf->charTable, sizeof (long)*csf->maxChar);
    F_append (fileName, csf->colors, csf->maxColor*sizeof(TRGBColor));
    F_append (fileName, csf->raw, csf->count*csf->size);
    return 0;
  }
  return -1;
}


int CSF_addRaw (char *draw, int dsizeX, int dsizeY,
                char *sraw, int ssizeX, int ssizeY, int one)
{
  int i,j, count;

  count = 0;
  for (j = 0; j < ssizeY; j+=dsizeY)
  {
    for (i = 0; i < ssizeX; i+=dsizeX)
    {
      LIN_get (sraw, i, j, ssizeX, ssizeY,
                  (char *)draw+count*dsizeX*dsizeY, dsizeX, dsizeX, dsizeY);
      count++;
      if (one)
        return 1;
    }
  }
  return count;
}

TCSF *CSF_createMulti (TCSF **csfList, int sizeX, int sizeY,
                       long charTable [256])
{
  int i;
  int total;
  long totalSize;
  TRGBColor pal [256];
  TCSF *csf;
  char *raw;
  long offset,count;

  memset (pal, 0, sizeof pal);
  i = 0;
  total = 0;
  totalSize = 0;
  while ((csf = csfList [i]) != NULL)
  {
    total++;
    totalSize += sizeX*((csf->sizeX / sizeX) + ((csf->sizeX % sizeX)!=0))*
                 sizeY*((csf->sizeY / sizeY) + ((csf->sizeY % sizeY)!=0));
    i++;
  }

  if (charTable)
  {
    totalSize+=sizeX*sizeY;
  }
  raw = (char *) _calloc (totalSize,1,NULL);

  if (raw)
  {
    if (charTable)
    {
      offset = sizeX*sizeY;
      count = 1;
    }
    else
    {
      offset = 0;
      count = 0;
    }
    for (i = 0; i < total; i++)
    {
      csf = csfList [i];
      CSF_updateColors (csf, pal, 1, 255);

      count += CSF_addRaw (raw+offset, sizeX, sizeY,
                              csf->raw,
                              csf->sizeX, csf->sizeY,(charTable) ? 1 : 0);
      offset = count*sizeX*sizeY;
    }
    csf = CSF_create (raw, sizeX, sizeY, pal, count, charTable);
    _free (raw);
    if (csf == NULL)
    {
      return NULL;
    }
  }
  else
  {
    return NULL;
  }
  return csf;
}


void CSF_put (TCSF *csf, int x, int y, int n)
{
  SVGA_putSpr (CSF_raw (csf,n), x, y, csf->sizeX, csf->sizeY, csf->sizeX);
}


int CSF_view (TCAR *carHandle, const char *fileName, TRGBColor pal [256],
              int n, int x, int y)
{
  int exitCode;
  TCSF *csf;

  csf = CSF_new (carHandle, fileName);
  if (csf)
  {
    CSF_updateColors (csf, pal, 1, 255);
    // do some adjust to center the image on the screen
    if (x == CSF_VIEW_CENTER)
    {
      x = (SVGA_maxX - csf->sizeX)/2;
    }

    if (y == CSF_VIEW_CENTER)
    {
      y = (SVGA_maxY - csf->sizeY)/2;
    }

    CSF_put (csf, x, y, n);
    CSF_delete (csf);
    exitCode = 0;
  }
  else
  {
    exitCode = -1;
  }
  return exitCode;
}

short *CSF_createMapBuffer (int sizeX, int sizeY, int tileX, int tileY, int *mapLenX, int *mapLenY)
{
  short *mapBuf;
  int i,j;
  int sLenX,sLenY;
  int lenX,lenY;

  lenX = _div (sizeX, tileX);
  lenY = _div (sizeY, tileY);
  sLenX = _div (SVGA_maxX, tileX);
  sLenY = _div (SVGA_maxY, tileY);
  *mapLenX = (lenX < sLenX) ? sLenX : lenX;
  *mapLenY = (lenY < sLenY) ? sLenY : lenY;

  mapBuf = (short *) _calloc (*mapLenX * *mapLenY, 2, NULL);

  if (mapBuf)
  {
    for (j = 0; j < lenY; j++)
    {
      for (i = 0; i < lenX; i++)
      {
        mapBuf [j* *mapLenX+i]=j*lenX+i+1;
      }
    }
  }
  else
  {
    *mapLenX = *mapLenY = 0;
  }
  return mapBuf;
}

char *CSF_createSprBuffer (char *raw, int sizeX, int sizeY, int tileX, int tileY)
{
  char *csfBuf;
  int lenX,lenY;

  lenX = _div (sizeX, tileX);
  lenY = _div (sizeY, tileY);
  csfBuf = (char *) _calloc (tileX*tileY,lenX*lenY+1, NULL);

  if (csfBuf)
  {
    CSF_addRaw (csfBuf+tileX*tileY, tileX, tileY, raw, sizeX, sizeY, 0);
  }
  return csfBuf;
}
