
#include <stdlib.h>
#include <dos.h>
#include <mem.h>
#include <conio.h>

#include "cgdk.h"
#include "svga.h"
#include "xvga.h"
#include "x256_.h"

#if !defined (__TURBOC__) && !defined (CGDK_TRIAL)

extern "C" {

extern void cdecl X256_movPlane (void *dst, void *src, int sizeX, int sizeY, int formatX);

extern void cdecl X256_getPlane (void *dst, void *src, int sizeX, int sizeY, int formatX);

extern void cdecl X256_movPlane2 (void *dst, void *dst2, void *src, int sizeX, int sizeY, int formatX);

extern void cdecl X256_movPlaneSpr (void *dst, void *src, int sizeX, int sizeY, int formatX);

extern void cdecl X256_movPlaneSpr2 (void *dst, void *dst2, void *src, int sizeX, int sizeY, int formatX);

}



//#include "x256c_.cpp"

#else

extern void cdecl X256_movPlane (void *dst, void *src, int sizeX, int sizeY, int formatX)
{
  int i, j;
  char *d = (char *) dst;
  char *s = (char *) src;

  for (j = 0; j < sizeY; j++)
  {
    for (i = 0; i < sizeX; i++)
    {
      d [i] = s [4*i];
    }
    s += formatX;
    d += XVGA_columns;
  }
}

extern void cdecl X256_getPlane (void *dst, void *src, int sizeX, int sizeY, int formatX)
{
  int i, j;
  char *d = (char *) dst;
  char *s = (char *) src;

  for (j = 0; j < sizeY; j++)
  {
    for (i = 0; i < sizeX; i++)
    {
      s [4*i] = d [i];
    }
    s += formatX;
    d += XVGA_columns;
  }
}

extern void cdecl X256_movPlane2 (void *dst, void *dst2, void *src, int sizeX, int sizeY, int formatX)
{
  int i, j;
  char *d = (char *) dst;
  char *d2 = (char *) dst2;
  char *s = (char *) src;

  for (j = 0; j < sizeY; j++)
  {
    for (i = 0; i < sizeX; i++)
    {
      d [i] = d2 [i] = s [4*i];
    }
    s += formatX;
    d += XVGA_columns;
    d2 += XVGA_columns;
  }
}

extern void cdecl X256_movPlaneSpr (void *dst, void *src, int sizeX, int sizeY, int formatX)
{
  int i, j;
  char *d = (char *) dst;
  char *s = (char *) src;

  for (j = 0; j < sizeY; j++)
  {
    for (i = 0; i < sizeX; i++)
    {
      if (s [4*i])
      {
        d [i] = s [4*i];
      }
    }
    s += formatX;
    d += XVGA_columns;
  }
}

extern void cdecl X256_movPlaneSpr2 (void *dst, void *dst2, void *src, int sizeX, int sizeY, int formatX)
{
  int i, j;
  char *d = (char *) dst;
  char *d2 = (char *) dst2;
  char *s = (char *) src;

  for (j = 0; j < sizeY; j++)
  {
    for (i = 0; i < sizeX; i++)
    {
      if (s [4*i])
      {
        d [i] = d2 [i] = s [4*i];
      }
    }
    s += formatX;
    d += XVGA_columns;
    d2 += XVGA_columns;    
  }
}



/*
void X256C_done (void)
{

}

void X256C_init (void)
{

}

void X256C_flush (void)
{
  SVGA_wait (1);
}

*/

#endif

void X256_clear (long color)
{
  outpw (SEQU_ADDR, 0x0F02);
  memset (SVGA_videoBuf, color, 0xFFFF);
  XVGA_writePlane = -1;
  XVGA_readPlane = -1;
}


#ifdef __TURBOC__

static void _DrawLineSpr2 (void *Dst, void *Src, int SizeX, int Plane, int limit)
{
  asm {
    push    ds
    push    es
  }

  _DS = FP_SEG (Src);
  _SI = FP_OFF (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _CX = SizeX;
  _AX = Plane;
  _BX = limit;

  asm {
    mov     dx,cx
    shr     cx,1
    shr     cx,1
    shl     cx,1
    shl     cx,1
    sub     dx,cx
    sub     dx,ax
    shr     cx,1
    shr     cx,1
    or      cx,cx
    je      DLS_endLoop
  }

  DLS_loop:

  asm {
    cmp     di,bx
    jna     DLS_getPixel
    sub     di,bx
  }

  DLS_getPixel:

  asm {
    mov     al,[si]
    or      ax,ax
    je      DLS_continue
  }
  DLS_savePixel:
  asm {
    mov     es:[di],al
    mov     es:[di+bx],al
  }
  DLS_continue:

  asm {
    inc     di
    add     si,4
    loop    DLS_loop
  }

  DLS_endLoop:

  asm {
    cmp     dx,cx //dh = current plane, dl = planes plus
    jle     DLS_return
    cmp     di,bx
    jna     DLS_lastPixel
    sub     di,bx
  }

  DLS_lastPixel:

  asm {
    mov     al,[si]
    or      ax,ax
    je      DLS_return
    mov     es:[di],al
    mov     es:[di+bx],al
  }

  DLS_return:

  asm {
    pop     es
    pop     ds
  }
}

static void _DrawLineWin2 (void *Dst, void *Src, int SizeX, int Plane, int limit)
{
  asm {
    push    ds
    push    es
  }

  _DS = FP_SEG (Src);
  _SI = FP_OFF (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _CX = SizeX;
  _AX = Plane;
  _BX = limit;

  asm {
    mov     dx,cx
    shr     cx,1
    shr     cx,1
    shl     cx,1
    shl     cx,1
    sub     dx,cx
    sub     dx,ax
    shr     cx,1
    shr     cx,1
    or      cx,cx
    je      DLW_endLoop
  }

  DLW_loop:

  asm {
    cmp     di,bx
    jna     DLS_getPixel
    sub     di,bx
  }

  DLS_getPixel:

  asm {
    mov     al,[si]
    mov     es:[di+bx],al
    mov     es:[di],al
    inc     di
    add     si,4
    loop    DLW_loop
  }

  DLW_endLoop:

  asm {
    cmp     dx,cx
    jle     DLW_return
    cmp     di,bx
    jna     DLS_lastPixel
    sub     di,bx
  }

  DLS_lastPixel:

  asm {
    mov     al,[si]
    mov     es:[di+bx],al
    mov     es:[di],al
  }

  DLW_return:

  asm {
    pop     es
    pop     ds
  }
}


static void _DrawLineSpr (void *Dst, void *Src, int SizeX, int Plane)
{
  asm {
      push ax
      push bx
      push cx
      push dx
      push si
      push di
      push bp

    push    ds
    push    es
  }

  _DS = FP_SEG (Src);
  _SI = FP_OFF (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _CX = SizeX;
  _DX = Plane;

  asm {
    xor     ax,ax
    push    cx
    shr     cx,1
    shr     cx,1
    push    cx
    jcxz    DLS_endLoop
  }

  DLS_loop:

  asm {
    lodsb
    or      ax,ax
    jne     DLS_savePixel
    inc     di
    jmp     DLS_continue
  }
  DLS_savePixel:
  asm {
    stosb
  }
  DLS_continue:

  asm {
    add     si,3
    loop    DLS_loop
  }

  DLS_endLoop:

  asm {
    pop     cx
    pop     ax
    shl     cx,1
    shl     cx,1
    sub     ax,cx
    cmp     dx,ax //dh = current plane, dl = planes plus
    jae     DLS_return
    lodsb
    or      ax,ax
    je      DLS_return
    stosb
  }

  DLS_return:

  asm {
    pop     es
    pop     ds
      pop bp
      pop di
      pop si
      pop dx
      pop cx
      pop bx
      pop ax

  }
}


static void _DrawLineWin (void *Dst, void *Src, int SizeX, int Plane)
{
  asm {
      push ax
      push bx
      push cx
      push dx
      push si
      push di
      push bp

    push    ds
    push    es
  }

  _DS = FP_SEG (Src);
  _SI = FP_OFF (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _CX = SizeX;
  _DX = Plane;

  asm {
    xor     ax,ax
    push    cx
    shr     cx,1
    shr     cx,1
    push    cx
    jcxz    DLW_endLoop
  }

  DLW_loop:

  asm {
    movsb
    add     si,3
    loop    DLW_loop
  }

  DLW_endLoop:

  asm {
    pop     cx
    pop     ax
    shl     cx,1
    shl     cx,1
    sub     ax,cx
    cmp     dx,ax //dh = current plane, dl = planes plus
    jae     DLW_return
    movsb
  }

  DLW_return:

  asm {
    pop     es
    pop     ds
      pop bp
      pop di
      pop si
      pop dx
      pop cx
      pop bx
      pop ax

  }
}




static void _DrawPlaneSpr (void *Dst, void *Src, int SizeX, int SizeY, int formatX,
		    char Plane, char Cols)
{
  asm {
      push ax
      push bx
      push cx
      push dx
      push si
      push di
      push bp

    push    bp
    push    ds
    push    es
  }

  _DS = FP_SEG (Src);
  _SI = FP_OFF (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _BX = SizeX;
  _CX = SizeY;
  _AH = Cols;
  _AL = Plane;
  _BP = formatX;

  asm {
    jcxz    S_Quit
    mov     dx,bx
    shr     bx,1
    shr     bx,1
    shl     bx,1
    shl     bx,1
    sub     dx,bx
    shr     bx,1
    shr     bx,1
    mov     dh,al
  }

  S_DrawY:

  asm {
    push    cx
    push    si
    push    di
    mov     cx,bx
    jcxz    S_EndLoop
  }

  S_StosbVLoop:

  asm {
    lodsb
    or      al,al
    jne     S_StosbVideo
    inc     di
    jmp     S_Continue
  }
  S_StosbVideo:
  asm {
    stosb
  }
  S_Continue:
  asm {

  }

  asm {
    add     si,3
    loop    S_StosbVLoop

  }

  S_EndLoop:

  asm {
    cmp     dh,dl //dh = current plane, dl = planes plus
    jae     S_IncSi
    lodsb
    or      al,al
    je      S_IncSi
    stosb
  }

  S_IncSi:

  asm {
//    mov     al,dh
//    xor     dh,dh
//    add     si,dx
//    mov     dh,al

    pop     di
    xor     al,al
    xchg    al,ah
    add     di,ax
    xchg    al,ah

    pop     si
    add     si,bp

    pop     cx
    loop    S_DrawY
  }

  S_Quit:

  asm {
    pop     es
    pop     ds
    pop     bp
      pop bp
      pop di
      pop si
      pop dx
      pop cx
      pop bx
      pop ax

  }
}

static void _DrawPlaneWin (void *Dst, void *Src, int SizeX, int SizeY, int formatX,
                    char Plane, char Cols)
{
  asm {
      push ax
      push bx
      push cx
      push dx
      push si
      push di
      push bp

    push    bp
    push    ds
    push    es
  }

  _DS = FP_SEG (Src);
  _SI = FP_OFF (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _BX = SizeX;
  _CX = SizeY;
  _AH = Cols;
  _AL = Plane;
  _BP = formatX;

  asm {
    jcxz    W_Quit
    mov     dx,bx
    shr     bx,1
    shr     bx,1
    shl     bx,1
    shl     bx,1
    sub     dx,bx
    shr     bx,1
    shr     bx,1
    mov     dh,al
  }

  W_DrawY:

  asm {
    push    cx
    push    si
    push    di
    mov     cx,bx
    jcxz    W_EndLoop
  }

  W_StosbVLoop:

  asm {
    movsb
    add     si,3
    loop    W_StosbVLoop
  }

  W_EndLoop:

  asm {
    cmp     dh,dl //dh = current plane, dl = planes plus
    jae     W_IncSi
    movsb
//    mov     al,[si]
//    or      ax,ax
//    je      W_IncSi
//    stosb
  }

  W_IncSi:

  asm {
//    mov     al,dh
//    xor     dh,dh
//    add     si,dx
//    mov     dh,al

    pop     di
    xor     al,al
    xchg    al,ah
    add     di,ax
    xchg    al,ah

    pop     si
    add     si,bp

    pop     cx
    loop    W_DrawY
  }

  W_Quit:

  asm {
    pop     es
    pop     ds
    pop     bp
      pop bp
      pop di
      pop si
      pop dx
      pop cx
      pop bx
      pop ax

  }
}


static void _ReadPlane (void *Dst,void *Src, int SizeX, int SizeY, int formatX,
                 char Plane, char Cols)
{
  asm {
      push ax
      push bx
      push cx
      push dx
      push si
      push di
      push bp

    push    bp
    push    ds
    push    es
  }

  _SI = FP_OFF (Src);
  _DS = FP_SEG (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _BX = SizeX;
  _CX = SizeY;
  _AL = Plane;
  _AH = Cols;
  _BP = formatX;

  asm {
    jcxz    R_Quit
    mov     dx,bx
    shr     bx,1
    shr     bx,1
    shl     bx,1
    shl     bx,1
    sub     dx,bx
    shr     bx,1
    shr     bx,1
    mov     dh,al
  }

  R_DrawY:

  asm {
    push    cx
    mov     cx,bx
    push    di
    push    si
    jcxz    R_EndLoop
  }

  R_StosbVLoop:

  asm {
    movsb
    add     di,3
    loop    R_StosbVLoop
  }

  R_EndLoop:

  asm {
    cmp     dh,dl
    jae     R_IncDi
    movsb

  }

  R_IncDi:

  asm {
//    mov     al,dh
//    xor     dh,dh
//    add     di,dx
//    mov     dh,al

    pop     si
    xor     al,al
    xchg    al,ah
    add     si,ax
    xchg    al,ah

    pop     di
    add     di,bp

    pop     cx
    loop    R_DrawY
  }

  R_Quit:

  asm {
    pop     es
    pop     ds
    pop     bp
      pop bp
      pop di
      pop si
      pop dx
      pop cx
      pop bx
      pop ax

  }
}


#else

/*
void _DrawLineSpr2 (void *Dst, void *Src, int SizeX, int plane, int limit);
#pragma aux _DrawLineSpr2 = \
"    mov     edx,ecx"\
"    shr     ecx,2"\
"    shl     ecx,2"\
"    sub     edx,ecx"\
"    sub     edx,eax"\
"    shr     ecx,2"\
"    or      ecx,ecx"\
"    je      DLS_endLoop"\
"DLS_loop:"\
"    cmp     di,bx"\
"    jna     DLS_getPixel"\
"    sub     edi,ebx"\
"DLS_getPixel:"\
"    mov     al,[esi]"\
"    or      eax,eax"\
"    je      DLS_continue"\
"DLS_savePixel:"\
"    mov     [edi],al"\
"    mov     [edi+ebx],al"\
"DLS_continue:"\
"    inc     edi"\
"    add     esi,4"\
"    loop    DLS_loop"\
"DLS_endLoop:"\
"    cmp     edx,ecx"\
"    jle     DLS_return"\
"    cmp     di,bx"\
"    jna     DLS_lastPixel"\
"    sub     edi,ebx"\
"DLS_lastPixel:"\
"    mov     al,[esi]"\
"    or      eax,eax"\
"    je      DLS_return"\
"    mov     [edi],al"\
"    mov     [edi+ebx],al"\
"DLS_return:"\
  parm [edi] [esi] [ecx] [eax] [ebx]\
  modify [eax ebx ecx edx esi edi]\

void _DrawLineWin2 (void *Dst, void *Src, int SizeX, int plane, int limit);
#pragma aux _DrawLineWin2 = \
"    mov     edx,ecx"\
"    shr     ecx,2"\
"    shl     ecx,2"\
"    sub     edx,ecx"\
"    sub     edx,eax"\
"    shr     ecx,2"\
"    or      ecx,ecx"\
"    je      DLS_endLoop"\
"DLS_loop:"\
"    cmp     di,bx"\
"    jna     DLS_getPixel"\
"    sub     edi,ebx"\
"DLS_getPixel:"\
"    mov     al,[esi]"\
"    mov     [edi],al"\
"    mov     [edi+ebx],al"\
"    inc     edi"\
"    add     esi,4"\
"    loop    DLS_loop"\
"DLS_endLoop:"\
"    cmp     edx,ecx"\
"    jle     DLS_return"\
"    cmp     di,bx"\
"    jna     DLS_lastPixel"\
"    sub     edi,ebx"\
"DLS_lastPixel:"\
"    mov     al,[esi]"\
"    mov     [edi],al"\
"    mov     [edi+ebx],al"\
"DLS_return:"\
  parm [edi] [esi] [ecx] [eax] [ebx]\
  modify [eax ebx ecx edx esi edi]\


void _DrawPlaneSpr (void *Dst, void *Src, int SizeX, int SizeY, int formatX, char Plane, char Cols);
#pragma aux _DrawPlaneSpr = \
"    jecxz    DPQuit"\
"    push    ebp"\
"    mov     ebp,edx"\
"    mov     edx,ebx"\
"    shr     ebx,2"\
"    shl     ebx,2"\
"    sub     edx,ebx"\
"    shr     ebx,2"\
"    mov     dh,al"\
\
"  PSDrawY:"\
"    push    ecx"\
"    mov     ecx,ebx"\
"    push    esi"\
"    push    edi"\
"    or      ecx,ecx"\
"    je      SVLEndLoop"\
\
"  StosbVLoop:"\
"    mov     al,[esi]"\
"    or      al,al"\
"    je      SVLContinue"\
"    mov     [edi],al"\
"  SVLContinue:"\
"    inc     di"\
"    add     esi,4"\
"    dec     ecx"\
"    or      ecx,ecx"\
"    jne     StosbVLoop"\
\
"  SVLEndLoop:"\
"    cmp     dh,dl"\
"    jae     _SVLIncSi"\
"    mov     al,[esi]"\
"    or      al,al"\
"    je      _SVLIncSi"\
"  _StosbVideo:"\
"    mov     [edi],al"\
\
"   _SVLIncSi:"\
\
"    pop     edi"\
"    xor     al,al"\
"    xchg    al,ah"\
"    add     di,ax"\
"    xchg    ah,al"\
"    pop     esi"\
"    add     esi,ebp"\
"    pop     ecx"\
"    loop    PSDrawY"\
"    pop     ebp"\
"  DPQuit:"\
  parm [edi] [esi] [ebx] [ecx] [edx] [al] [ah]\
  modify [eax ebx ecx edx esi edi]\

  /*
  _DS = FP_SEG (Src);
  _SI = FP_OFF (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _BX = SizeX;
  _CX = SizeY;
  _DX = Cols;
  _AH = Type;
  */
/*
void _DrawPlaneWin (void *Dst, void *Src, int SizeX, int SizeY, int formatX, char Plane, char Cols);
#pragma aux _DrawPlaneWin = \
"    jecxz    DPQuit"\
"    push    ebp"\
"    mov     ebp,edx"\
"    mov     edx,ebx"\
"    shr     ebx,2"\
"    shl     ebx,2"\
"    sub     edx,ebx"\
"    shr     ebx,2"\
"    mov     dh,al"\
\
"  PSDrawY:"\
"    push    ecx"\
"    mov     ecx,ebx"\
"    push    esi"\
"    push    edi"\
"    jecxz   SVLEndLoop"\
\
"  StosbVLoop:"\
"    mov     al,[esi]"\
"    mov     [edi],al"\
"    inc     di"\
"    add     esi,4"\
"    loop    StosbVLoop"\
\
"  SVLEndLoop:"\
"    cmp     dh,dl"\
"    jae     _SVLIncSi"\
"    mov     al,[esi]"\
"    mov     [edi],al"\
\
"   _SVLIncSi:"\
\
"    pop     edi"\
"    xor     al,al"\
"    xchg    al,ah"\
"    add     di,ax"\
"    xchg    ah,al"\
"    pop     esi"\
"    add     esi,ebp"\
"    pop     ecx"\
"    loop    PSDrawY"\
"    pop     ebp"\
"  DPQuit:"\
  parm [edi] [esi] [ebx] [ecx] [edx] [al] [ah]\
  modify [eax ebx ecx edx esi edi]\

  /*
  _DS = FP_SEG (Src);
  _SI = FP_OFF (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _BX = SizeX;
  _CX = SizeY;
  _DX = Cols;
  _AH = Type;
  */

/*
void _ReadPlane (void *Dst,void *Src, int SizeX, int SizeY, int formatX, char Plane, char Cols);
#pragma aux _ReadPlane = \
\
"    jecxz    RPQuit"\
"    push    ebp"\
"    mov     ebp,edx"\
"    mov     edx,ebx"\
"    shr     ebx,2"\
"    shl     ebx,2"\
"    sub     edx,ebx"\
"    shr     ebx,2"\
"    mov     dh,al"\
\
"  GSDrawY:"\
\
"    push    ecx"\
"    mov     ecx,ebx"\
"    push    edi"\
"    push    esi"\
"    jecxz    SVLEndLoop"\
\
"  StosbVLoop:"\
"    mov     al,[esi]"\
"    mov     [edi],al"\
"    inc     si"\
"    add     edi,4"\
"    loop    StosbVLoop"\
"  SVLEndLoop:"\
"    cmp     dh,dl"\
"    jae     SVLIncDi"\
"    mov     al,[esi]"\
"    mov     [edi],al"\
\
"  SVLIncDi:"\
"    pop     esi"\
"    xor     al,al"\
"    xchg    al,ah"\
"    add     si,ax"\
"    xchg    ah,al"\
\
"    pop     edi"\
"    add     edi,ebp"\
\
"    pop     ecx"\
"    loop    GSDrawY"\
"    pop     ebp"\
"  RPQuit:"\
\
  parm [edi] [esi] [ebx] [ecx] [edx] [al] [ah]\
  modify [eax ebx ecx edx esi edi]\

  /*
  _SI = FP_OFF (Src);
  _DS = FP_SEG (Src);
  _ES = FP_SEG (Dst);
  _DI = FP_OFF (Dst);
  _BX = SizeX;
  _CX = SizeY;
  _DX = Cols;
  */

#endif



#ifdef __TURBOC__

/*
void X256_putWin2 (void *Spr,int X,int Y,int SIZE_X,int SIZE_Y, int formatX)
{
  int i,j;
  int imax;
  int curplane;
  unsigned offset;
  unsigned vgaOffset;
  unsigned sprOffset;
  unsigned limit=SVGA_virtualD4;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  curplane = X % 4;
  X /= 4;
  offset = X+Y*XVGA_columns;

  imax = (SIZE_X < 4) ? SIZE_X : 4;
  for (i = 0; i < imax ; i++)
  {
    vgaOffset = offset;
    sprOffset = i;
    XVGA_setWritePlane (curplane);
    for (j = 0; j < SIZE_Y; j++)
    {
      _DrawLineWin2 ((char *)SVGA_videoBuf+vgaOffset, (char *)Spr+sprOffset, SIZE_X, i, limit);
      vgaOffset += XVGA_columns;
      sprOffset += formatX;
    }
    curplane++;
    if (curplane == 4)
    {
      curplane = 0;
      offset++;
    }
  }
}

void X256_putSpr2 (void *Spr,int X,int Y,int SIZE_X,int SIZE_Y, int formatX)
{
  int i,j;
  int imax;
  int curplane;
  unsigned offset;
  unsigned vgaOffset;
  unsigned sprOffset;
  unsigned limit=SVGA_virtualD4;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  curplane = X % 4;
  X /= 4;
  offset = X+Y*XVGA_columns;

  imax = (SIZE_X < 4) ? SIZE_X : 4;
  for (i = 0; i < imax ; i++)
  {
    vgaOffset = offset;
    sprOffset = i;
    XVGA_setWritePlane (curplane);
    for (j = 0; j < SIZE_Y; j++)
    {
      _DrawLineSpr2 ((char *)SVGA_videoBuf+vgaOffset, (char *)Spr+sprOffset, SIZE_X, i, limit);
      vgaOffset += XVGA_columns;
      sprOffset += formatX;
    }
    curplane++;
    if (curplane == 4)
    {
      curplane = 0;
      offset++;
    }
  }
}

void X256_putWin (void *Spr,int X,int Y,int SIZE_X,int SIZE_Y, int formatX)
{
  int i, imax;
  int curplane;
  unsigned int offset;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  curplane = X % 4;
  X /= 4;

  offset = X+Y*XVGA_columns;
  imax = (SIZE_X < 4) ? SIZE_X : 4;
  for (i = 0; i < imax; i++)
  {
    XVGA_setWritePlane (curplane);
    _DrawPlaneWin ((char *) SVGA_videoBuf +offset,
                   (char *)Spr+ i, SIZE_X, SIZE_Y, formatX, i, XVGA_columns);
    curplane++;
    if (curplane == 4)
    {
      curplane = 0;
      offset++;
    }
  }
}

void X256_putSpr (void *Spr,int X,int Y,int SIZE_X,int SIZE_Y, int formatX)
{
  int i, imax;
  int curplane;
  unsigned int offset;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  curplane = X % 4;
  X = X >> 2;

  offset = X+Y*XVGA_columns;
  imax = (SIZE_X < 4) ? SIZE_X : 4;
  for (i = 0; i < imax; i++)
  {
    XVGA_setWritePlane (curplane);
    _DrawPlaneSpr ((char *) SVGA_videoBuf +offset,
                   (char *)Spr+ i, SIZE_X, SIZE_Y, formatX, i, XVGA_columns);
    curplane++;
    if (curplane == 4)
    {
      curplane = 0;
      offset++;
    }
  }
}

void X256_getWin (void *Spr, int X, int Y, int SIZE_X, int SIZE_Y,int formatX)
{
  int i;
  int curplane;
  int imax;
  unsigned offset;

  if (Spr != NULL)
  {
    X += SVGA_curX;
    Y += SVGA_curY;
    SVGA_virtual2Real (&X, &Y);

    curplane = X % 4;
    X /= 4;
    imax = (SIZE_X < 4) ? SIZE_X : 4;

    offset = X + Y*XVGA_columns;

    for (i = 0; i < imax; i++)
    {
      XVGA_setReadPlane (curplane);
      _ReadPlane ( (char *) Spr+i,
                   (char *) SVGA_videoBuf + offset,
                   SIZE_X, SIZE_Y, formatX, i, XVGA_columns);
      curplane++;
      if (curplane == 4)
      {
        curplane = 0;
        offset++;
      }
    }
  }
}
*/

//#else
#endif


void X256_getWin (void *spr, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int planeCount;
  int plane [4];
  int i;
  int planei;
  int x4,x,y;
  int lenX;
  char *dstOff;
  char *sprOff;
  int sizeX, sizeY, restX, restY;

  planeCount = (SIZE_X > 4) ? 4 : SIZE_X;

  for (i = 0; i < planeCount; i++)
  {
    if (((X+i) % 4) == XVGA_readPlane)
    {
      plane [i] = plane [0];
      plane [0] = i;
      for (i++; i < planeCount; i++)
      {
        plane [i] = i;
      }
    }
    else
    {
      plane [i] = i;
    }
  }

  for (i = 0; i < planeCount; i++)
  {
    planei = plane [i];
    sprOff = (char *) spr + planei;
    x = SVGA_curX + X + planei;
    y = SVGA_curY + Y;
    SVGA_virtual2Real (&x, &y);
    XVGA_setReadPlane (x % 4);
    x4 = x/4;
    dstOff = (char *)SVGA_videoBuf + y * XVGA_columns + x4;
    lenX = SIZE_X / 4;
    if ((SIZE_X % 4) > planei)
      lenX++;
    sizeY = SVGA_virtualY - y;
    if (sizeY <= SIZE_Y)
    {
      restY = SIZE_Y - sizeY;
      if ((XVGA_columns - x4) < lenX)
      {
        sizeY--;
        restX = XVGA_columns - x4;
      }
      else
        restX = 0;

      if (sizeY)
      {
        X256_getPlane (dstOff, sprOff, lenX, sizeY, formatX);
        sprOff += sizeY * formatX;
        dstOff += sizeY * XVGA_columns;
      }

      if (restX)
      {
        X256_getPlane (dstOff, sprOff, restX, 1, formatX);

        dstOff = (char *) SVGA_videoBuf;

        X256_getPlane (dstOff, sprOff + 4 * restX, lenX - restX, 1, formatX);

        sprOff += formatX;
        dstOff += x4;
      }
      else
        dstOff = (char *) SVGA_videoBuf + x4;

      if (restY > 0)
      {
        X256_getPlane (dstOff, sprOff, lenX, restY, formatX);
      }
    }
    else
    {
      X256_getPlane (dstOff, sprOff, lenX, SIZE_Y, formatX);
    }
  }
}





void X256_putWin (void *spr, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int planeCount;
  int plane [4];
  int i;
  int planei;
  int x4,x,y;
  int lenX;
  char *dstOff;
  char *sprOff;
  int sizeX, sizeY, restX, restY;

  planeCount = (SIZE_X > 4) ? 4 : SIZE_X;

  for (i = 0; i < planeCount; i++)
  {
    if (((X+i) % 4) == XVGA_writePlane)
    {
      plane [i] = plane [0];
      plane [0] = i;
      for (i++; i < planeCount; i++)
      {
        plane [i] = i;
      }
    }
    else
    {
      plane [i] = i;
    }
  }

  for (i = 0; i < planeCount; i++)
  {
    planei = plane [i];
    sprOff = (char *) spr + planei;
    x = SVGA_curX + X + planei;
    y = SVGA_curY + Y;
    SVGA_virtual2Real (&x, &y);
    XVGA_setWritePlane (x % 4);
    x4 = x/4;
    dstOff = (char *)SVGA_videoBuf + y * XVGA_columns + x4;
    lenX = SIZE_X / 4;
    if ((SIZE_X % 4) > planei)
      lenX++;
    sizeY = SVGA_virtualY - y;
    if (sizeY <= SIZE_Y)
    {
      restY = SIZE_Y - sizeY;
      if ((XVGA_columns - x4) < lenX)
      {
        sizeY--;
        restX = XVGA_columns - x4;
      }
      else
        restX = 0;

      if (sizeY)
      {
        X256_movPlane (dstOff, sprOff, lenX, sizeY, formatX);
        sprOff += sizeY * formatX;
        dstOff += sizeY * XVGA_columns;
      }

      if (restX)
      {
        X256_movPlane (dstOff, sprOff, restX, 1, formatX);

        dstOff = (char *) SVGA_videoBuf;

        X256_movPlane (dstOff, sprOff + 4 * restX, lenX - restX, 1, formatX);

        sprOff += formatX;
        dstOff += x4;
      }
      else
        dstOff = (char *) SVGA_videoBuf + x4;

      if (restY > 0)
      {
        X256_movPlane (dstOff, sprOff, lenX, restY, formatX);
      }
    }
    else
    {
      X256_movPlane (dstOff, sprOff, lenX, SIZE_Y, formatX);
    }
  }
}

void X256_putWin2 (void *spr, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int planeCount;
  int plane [4];
  int i;
  int planei;
  int x4,x,y;
  int lenX;
  char *dstOff;
  char *sprOff;
  int sizeX, sizeY, restX, restY;
  int limit=SVGA_virtualD4;

  planeCount = (SIZE_X > 4) ? 4 : SIZE_X;

  for (i = 0; i < planeCount; i++)
  {
    if (((X+i) % 4) == XVGA_writePlane)
    {
      plane [i] = plane [0];
      plane [0] = i;
      for (i++; i < planeCount; i++)
      {
        plane [i] = i;
      }
    }
    else
    {
      plane [i] = i;
    }
  }

  for (i = 0; i < planeCount; i++)
  {
    planei = plane [i];
    sprOff = (char *) spr + planei;
    x = SVGA_curX + X + planei;
    y = SVGA_curY + Y;
    SVGA_virtual2Real (&x, &y);
    XVGA_setWritePlane (x % 4);
    x4 = x/4;
    dstOff = (char *)SVGA_videoBuf + y * XVGA_columns + x4;
    lenX = SIZE_X / 4;
    if ((SIZE_X % 4) > planei)
      lenX++;
    sizeY = SVGA_virtualY - y;
    if (sizeY <= SIZE_Y)
    {
      restY = SIZE_Y - sizeY;
      if ((XVGA_columns - x4) < lenX)
      {
        sizeY--;
        restX = XVGA_columns - x4;
      }
      else
        restX = 0;

      if (sizeY)
      {
        X256_movPlane2 (dstOff, dstOff + limit, sprOff, lenX, sizeY, formatX);
        sprOff += sizeY * formatX;
        dstOff += sizeY * XVGA_columns;
      }

      if (restX)
      {
        X256_movPlane2 (dstOff, dstOff + limit, sprOff, restX, 1, formatX);

        dstOff = (char *) SVGA_videoBuf;

        X256_movPlane2 (dstOff, dstOff + limit, sprOff + 4 * restX, lenX - restX, 1, formatX);

        sprOff += formatX;
        dstOff += x4;
      }
      else
        dstOff = (char *) SVGA_videoBuf + x4;

      if (restY > 0)
      {
        X256_movPlane2 (dstOff, dstOff + limit, sprOff, lenX, restY, formatX);
      }
    }
    else
    {
      X256_movPlane2 (dstOff, dstOff + limit, sprOff, lenX, SIZE_Y, formatX);
    }
  }
}



void X256_putSpr (void *spr, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int planeCount;
  int plane [4];
  int i;
  int planei;
  int x4,x,y;
  int lenX;
  char *dstOff;
  char *sprOff;
  int sizeX, sizeY, restX, restY;

  planeCount = (SIZE_X > 4) ? 4 : SIZE_X;

  for (i = 0; i < planeCount; i++)
  {
    if (((X+i) % 4) == XVGA_writePlane)
    {
      plane [i] = plane [0];
      plane [0] = i;
      for (i++; i < planeCount; i++)
      {
        plane [i] = i;
      }
    }
    else
    {
      plane [i] = i;
    }
  }

  for (i = 0; i < planeCount; i++)
  {
    planei = plane [i];
    sprOff = (char *) spr + planei;
    x = SVGA_curX + X + planei;
    y = SVGA_curY + Y;
    SVGA_virtual2Real (&x, &y);
    XVGA_setWritePlane (x % 4);
    x4 = x/4;
    dstOff = (char *)SVGA_videoBuf + y * XVGA_columns + x4;
    lenX = SIZE_X / 4;
    if ((SIZE_X % 4) > planei)
      lenX++;
    sizeY = SVGA_virtualY - y;
    if (sizeY <= SIZE_Y)
    {
      restY = SIZE_Y - sizeY;
      if ((XVGA_columns - x4) < lenX)
      {
        sizeY--;
        restX = XVGA_columns - x4;
      }
      else
        restX = 0;

      if (sizeY)
      {
        X256_movPlaneSpr (dstOff, sprOff, lenX, sizeY, formatX);
        sprOff += sizeY * formatX;
        dstOff += sizeY * XVGA_columns;
      }

      if (restX)
      {
        X256_movPlaneSpr (dstOff, sprOff, restX, 1, formatX);

        dstOff = (char *) SVGA_videoBuf;

        X256_movPlaneSpr (dstOff, sprOff + 4 * restX, lenX - restX, 1, formatX);

        sprOff += formatX;
        dstOff += x4;
      }
      else
        dstOff = (char *) SVGA_videoBuf + x4;

      if (restY > 0)
      {
        X256_movPlaneSpr (dstOff, sprOff, lenX, restY, formatX);
      }
    }
    else
    {
      X256_movPlaneSpr (dstOff, sprOff, lenX, SIZE_Y, formatX);
    }
  }
}

void X256_putSpr2 (void *spr, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int planeCount;
  int plane [4];
  int i;
  int planei;
  int x4,x,y;
  int lenX;
  char *dstOff;
  char *sprOff;
  int sizeX, sizeY, restX, restY;
  int limit=SVGA_virtualD4;

  planeCount = (SIZE_X > 4) ? 4 : SIZE_X;

  for (i = 0; i < planeCount; i++)
  {
    if (((X+i) % 4) == XVGA_writePlane)
    {
      plane [i] = plane [0];
      plane [0] = i;
      for (i++; i < planeCount; i++)
      {
        plane [i] = i;
      }
    }
    else
    {
      plane [i] = i;
    }
  }

  for (i = 0; i < planeCount; i++)
  {
    planei = plane [i];
    sprOff = (char *) spr + planei;
    x = SVGA_curX + X + planei;
    y = SVGA_curY + Y;
    SVGA_virtual2Real (&x, &y);
    XVGA_setWritePlane (x % 4);
    x4 = x/4;
    dstOff = (char *)SVGA_videoBuf + y * XVGA_columns + x4;
    lenX = SIZE_X / 4;
    if ((SIZE_X % 4) > planei)
      lenX++;
    sizeY = SVGA_virtualY - y;
    if (sizeY <= SIZE_Y)
    {
      restY = SIZE_Y - sizeY;
      if ((XVGA_columns - x4) < lenX)
      {
        sizeY--;
        restX = XVGA_columns - x4;
      }
      else
        restX = 0;

      if (sizeY)
      {
        X256_movPlaneSpr2 (dstOff, dstOff+limit, sprOff, lenX, sizeY, formatX);
        sprOff += sizeY * formatX;
        dstOff += sizeY * XVGA_columns;
      }

      if (restX)
      {
        X256_movPlaneSpr2 (dstOff, dstOff+limit, sprOff, restX, 1, formatX);

        dstOff = (char *) SVGA_videoBuf;

        X256_movPlaneSpr2 (dstOff, dstOff+limit, sprOff + 4 * restX, lenX - restX, 1, formatX);

        sprOff += formatX;
        dstOff += x4;
      }
      else
        dstOff = (char *) SVGA_videoBuf + x4;

      if (restY > 0)
      {
        X256_movPlaneSpr2 (dstOff, dstOff+limit, sprOff, lenX, restY, formatX);
      }
    }
    else
    {
      X256_movPlaneSpr2 (dstOff, dstOff+limit, sprOff, lenX, SIZE_Y, formatX);
    }
  }
}





/*


void X256_putWin2 (void *spr, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int planeCount;
  int plane [4];
  int cplane;
  int i;
  int planei;
  int x4,x,y;
  int lenX;
  char *dstOff;
  char *sprOff;
  int sizeX, sizeY, restX, restY;
  int limit=SVGA_virtualD4;

  planeCount = (SIZE_X > 4) ? 4 : SIZE_X;

  for (i = 0; i < planeCount; i++)
  {
    if (((X+i) % 4) == XVGA_writePlane)
    {
      plane [i] = plane [0];
      plane [0] = i;
      for (i++; i < planeCount; i++)
      {
        plane [i] = i;
      }
    }
    else
    {
      plane [i] = i;
    }
  }

  for (i = 0; i < planeCount; i++)
  {
    planei = plane [i];
    sprOff = (char *) spr + planei;
    x = SVGA_curX + X + planei;
    y = SVGA_curY + Y;
    SVGA_virtual2Real (&x, &y);
    cplane = x % 4;
    x4 = x/4;
    dstOff = (char *)SVGA_videoBuf + y * XVGA_columns + x4;
    lenX = SIZE_X / 4;
    if ((SIZE_X % 4) > planei)
      lenX++;
    sizeY = SVGA_virtualY - y;
    if (sizeY <= SIZE_Y)
    {
      restY = SIZE_Y - sizeY;
      if ((XVGA_columns - x4) < lenX)
      {
        sizeY--;
        restX = XVGA_columns - x4;
      }
      else
        restX = 0;

      if (sizeY)
      {
        X256C_putPlane2 (cplane, dstOff, dstOff + limit, sprOff, lenX, sizeY, formatX);
        sprOff += sizeY * formatX;
        dstOff += sizeY * XVGA_columns;
      }

      if (restX)
      {
        X256C_putPlane2 (cplane, dstOff, dstOff + limit, sprOff, restX, 1, formatX);

        dstOff = (char *) SVGA_videoBuf;

        X256C_putPlane2 (cplane, dstOff, dstOff + limit, sprOff + 4 * restX, lenX - restX, 1, formatX);

        sprOff += formatX;
        dstOff += x4;
      }
      else
        dstOff = (char *) SVGA_videoBuf + x4;

      if (restY > 0)
      {
        X256C_putPlane2 (cplane, dstOff, dstOff + limit, sprOff, lenX, restY, formatX);
      }
    }
    else
    {
      X256C_putPlane2 (cplane, dstOff, dstOff + limit, sprOff, lenX, SIZE_Y, formatX);
    }
  }

}



void X256_putSpr (void *spr, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{

  int planeCount;
  int plane [4];
  int cplane;
  int i;
  int planei;
  int x4,x,y;
  int lenX;
  char *dstOff;
  char *sprOff;
  int sizeX, sizeY, restX, restY;

  planeCount = (SIZE_X > 4) ? 4 : SIZE_X;

  for (i = 0; i < planeCount; i++)
  {
    if (((X+i) % 4) == XVGA_writePlane)
    {
      plane [i] = plane [0];
      plane [0] = i;
      for (i++; i < planeCount; i++)
      {
        plane [i] = i;
      }
    }
    else
    {
      plane [i] = i;
    }
  }

  for (i = 0; i < planeCount; i++)
  {
    planei = plane [i];
    sprOff = (char *) spr + planei;
    x = SVGA_curX + X + planei;
    y = SVGA_curY + Y;
    SVGA_virtual2Real (&x, &y);
    cplane = x % 4;
    XVGA_setWritePlane (cplane);
    x4 = x/4;
    dstOff = (char *)SVGA_videoBuf + y * XVGA_columns + x4;
    lenX = SIZE_X / 4;
    if ((SIZE_X % 4) > planei)
      lenX++;
    sizeY = SVGA_virtualY - y;
    if (sizeY <= SIZE_Y)
    {
      restY = SIZE_Y - sizeY;
      if ((XVGA_columns - x4) < lenX)
      {
        sizeY--;
        restX = XVGA_columns - x4;
      }
      else
        restX = 0;

      if (sizeY)
      {
        X256C_putPlaneSpr (cplane, dstOff, sprOff, lenX, sizeY, formatX);
        sprOff += sizeY * formatX;
        dstOff += sizeY * XVGA_columns;
      }

      if (restX)
      {
        X256C_putPlaneSpr (cplane, dstOff, sprOff, restX, 1, formatX);

        dstOff = (char *) SVGA_videoBuf;

        X256C_putPlaneSpr (cplane, dstOff, sprOff + 4 * restX, lenX - restX, 1, formatX);

        sprOff += formatX;
        dstOff += x4;
      }
      else
        dstOff = (char *) SVGA_videoBuf + x4;

      if (restY > 0)
      {
        X256C_putPlaneSpr (cplane, dstOff, sprOff, lenX, restY, formatX);
      }
    }
    else
    {
      X256C_putPlaneSpr (cplane, dstOff, sprOff, lenX, SIZE_Y, formatX);
    }
  }

}

void X256_putWin (void *spr, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int planeCount;
  int plane [4];
  int cplane;
  int i;
  int planei;
  int x4,x,y;
  int lenX;
  char *dstOff;
  char *sprOff;
  int sizeX, sizeY, restX, restY;

  planeCount = (SIZE_X > 4) ? 4 : SIZE_X;

  for (i = 0; i < planeCount; i++)
  {
    if (((X+i) % 4) == XVGA_writePlane)
    {
      plane [i] = plane [0];
      plane [0] = i;
      for (i++; i < planeCount; i++)
      {
        plane [i] = i;
      }
    }
    else
    {
      plane [i] = i;
    }
  }

  for (i = 0; i < planeCount; i++)
  {
    planei = plane [i];
    sprOff = (char *) spr + planei;
    x = SVGA_curX + X + planei;
    y = SVGA_curY + Y;
    SVGA_virtual2Real (&x, &y);
    cplane = x % 4;
    XVGA_setWritePlane (cplane);
    x4 = x/4;
    dstOff = (char *)SVGA_videoBuf + y * XVGA_columns + x4;
    lenX = SIZE_X / 4;
    if ((SIZE_X % 4) > planei)
      lenX++;
    sizeY = SVGA_virtualY - y;
    if (sizeY <= SIZE_Y)
    {
      restY = SIZE_Y - sizeY;
      if ((XVGA_columns - x4) < lenX)
      {
        sizeY--;
        restX = XVGA_columns - x4;
      }
      else
        restX = 0;

      if (sizeY)
      {
        X256C_putPlane (cplane, dstOff, sprOff, lenX, sizeY, formatX);
        sprOff += sizeY * formatX;
        dstOff += sizeY * XVGA_columns;
      }

      if (restX)
      {
        X256C_putPlane (cplane, dstOff, sprOff, restX, 1, formatX);

        dstOff = (char *) SVGA_videoBuf;

        X256C_putPlane (cplane, dstOff, sprOff + 4 * restX, lenX - restX, 1, formatX);

        sprOff += formatX;
        dstOff += x4;
      }
      else
        dstOff = (char *) SVGA_videoBuf + x4;

      if (restY > 0)
      {
        X256C_putPlane (cplane, dstOff, sprOff, lenX, restY, formatX);
      }
    }
    else
    {
      X256C_putPlane (cplane, dstOff, sprOff, lenX, SIZE_Y, formatX);
    }
  }
}

void X256_putSpr2 (void *spr, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int planeCount;
  int plane [4];
  int i;
  int planei;
  int cplane;
  int x4,x,y;
  int lenX;
  char *dstOff;
  char *sprOff;
  int sizeX, sizeY, restX, restY;
  int limit=SVGA_virtualD4;

  planeCount = (SIZE_X > 4) ? 4 : SIZE_X;

  for (i = 0; i < planeCount; i++)
  {
    if (((X+i) % 4) == XVGA_writePlane)
    {
      plane [i] = plane [0];
      plane [0] = i;
      for (i++; i < planeCount; i++)
      {
        plane [i] = i;
      }
    }
    else
    {
      plane [i] = i;
    }
  }

  for (i = 0; i < planeCount; i++)
  {
    planei = plane [i];
    sprOff = (char *) spr + planei;
    x = SVGA_curX + X + planei;
    y = SVGA_curY + Y;
    SVGA_virtual2Real (&x, &y);
    cplane = x % 4;
    XVGA_setWritePlane (cplane);
    x4 = x/4;
    dstOff = (char *)SVGA_videoBuf + y * XVGA_columns + x4;
    lenX = SIZE_X / 4;
    if ((SIZE_X % 4) > planei)
      lenX++;
    sizeY = SVGA_virtualY - y;
    if (sizeY <= SIZE_Y)
    {
      restY = SIZE_Y - sizeY;
      if ((XVGA_columns - x4) < lenX)
      {
        sizeY--;
        restX = XVGA_columns - x4;
      }
      else
        restX = 0;

      if (sizeY)
      {
        X256C_putPlaneSpr2 (cplane, dstOff, dstOff+limit, sprOff, lenX, sizeY, formatX);
        sprOff += sizeY * formatX;
        dstOff += sizeY * XVGA_columns;
      }

      if (restX)
      {
        X256C_putPlaneSpr2 (cplane, dstOff, dstOff+limit, sprOff, restX, 1, formatX);

        dstOff = (char *) SVGA_videoBuf;

        X256C_putPlaneSpr2 (cplane, dstOff, dstOff+limit, sprOff + 4 * restX, lenX - restX, 1, formatX);

        sprOff += formatX;
        dstOff += x4;
      }
      else
        dstOff = (char *) SVGA_videoBuf + x4;

      if (restY > 0)
      {
        X256C_putPlaneSpr2 (cplane, dstOff, dstOff+limit, sprOff, lenX, restY, formatX);
      }
    }
    else
    {
      X256C_putPlaneSpr2 (cplane, dstOff, dstOff+limit, sprOff, lenX, SIZE_Y, formatX);
    }
  }

}

*/



//#endif

/*
void X256_movPixel (int x, int y, long color)
{
  if (SVGA_doubleBuffer)
  {
    X256_movWin2_ (&color, x, y, 1, 1, 1);
  }
  else
  {
    X256_movWin_ (&color, x, y, 1, 1, 1);
  }
}
*/

void X256_putPixel (int x, int y, long color)
{
  SVGA_putWin_ (&color, x, y, 1, 1, 1);
}

long X256_getPixel (int x, int y)
{
  long t=0;
  X256_getWin (&t, x, y, 1, 1, 1);
  return t;
}

